using System;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Web.Security;

namespace XmlProviders.Data
{
    /// <summary>
    /// 
    /// </summary>
    [Serializable()]
    public class User
    {
        private string _userName;
        private string _email;
        private byte[] _password;
        private bool _locked;
        private bool _active;
        private string _challengeQuestion;
        private string _challengeAnswer;
        private string _description;
        private Guid _userKey;

        private DateTime _lastLogin;
        private DateTime _lastActivity;
        private DateTime _lastPasswordChanged;
        private DateTime _lastLockedOut;
        private DateTime _createdOn;

        /// <summary>
        /// Gets or sets the key.
        /// </summary>
        /// <value>The key.</value>
        [XmlAttribute("id")]
        public Guid Key
        {
            get { return _userKey; }
            set { _userKey = value; }
        }

        /// <summary>
        /// Gets or sets the name of the user.
        /// </summary>
        /// <value>The name of the user.</value>
        [XmlAttribute("username")]
        public string UserName
        {
            get { return _userName; }
            set { _userName = value; }
        }

        /// <summary>
        /// Gets or sets the email.
        /// </summary>
        /// <value>The email.</value>
        [XmlAttribute("email")]
        public string Email
        {
            get { return _email; }
            set { _email = value; }
        }

        /// <summary>
        /// Gets or sets the description.
        /// </summary>
        /// <value>The description.</value>
        [XmlElement("description")]
        public string Description
        {
            get { return _description; }
            set { _description = value; }
        }

        /// <summary>
        /// Gets or sets the password.
        /// </summary>
        /// <value>The password.</value>
        [XmlElement(typeof(byte[]), DataType = "base64Binary", ElementName = "password")]
        public byte[] Password
        {
            get { return _password; }
            set { _password = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is locked.
        /// </summary>
        /// <value><c>true</c> if this instance is locked; otherwise, <c>false</c>.</value>
        [XmlAttribute("locked")]
        public bool IsLocked
        {
            get { return _locked; }
            set { _locked = value; }
        }

        /// <summary>
        /// Gets or sets the challenge question.
        /// </summary>
        /// <value>The challenge question.</value>
        [XmlElement("question")]
        public string ChallengeQuestion
        {
            get { return _challengeQuestion; }
            set { _challengeQuestion = value; }
        }

        /// <summary>
        /// Gets or sets the challenge answer.
        /// </summary>
        /// <value>The challenge answer.</value>
        [XmlElement("answer")]
        public string ChallengeAnswer
        {
            get { return _challengeAnswer; }
            set { _challengeAnswer = value; }
        }

        /// <summary>
        /// Gets or sets the last logged in.
        /// </summary>
        /// <value>The last logged in.</value>
        [XmlAttribute("lastLogin")]
        public DateTime LastLoggedIn
        {
            get { return _lastLogin; }
            set { _lastLogin = value; }
        }

        /// <summary>
        /// Gets or sets the last activity.
        /// </summary>
        /// <value>The last activity.</value>
        [XmlAttribute("lastActivity")]
        public DateTime LastActivity
        {
            get { return _lastActivity; }
            set { _lastActivity = value; }
        }

        /// <summary>
        /// Gets or sets the last password change.
        /// </summary>
        /// <value>The last password change.</value>
        [XmlAttribute("lastPasswordChange")]
        public DateTime LastPasswordChange
        {
            get { return _lastPasswordChanged; }
            set { _lastPasswordChanged = value; }
        }

        /// <summary>
        /// Gets or sets the last locked out.
        /// </summary>
        /// <value>The last locked out.</value>
        [XmlAttribute("lastLockedOut")]
        public DateTime LastLockedOut
        {
            get { return _lastLockedOut; }
            set { _lastLockedOut = value; }
        }

        /// <summary>
        /// Gets or sets the created on.
        /// </summary>
        /// <value>The created on.</value>
        [XmlAttribute("createdOn")]
        public DateTime CreatedOn
        {
            get { return _createdOn; }
            set { _createdOn = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is active.
        /// </summary>
        /// <value><c>true</c> if this instance is active; otherwise, <c>false</c>.</value>
        [XmlAttribute("active")]
        public bool IsActive
        {
            get { return _active; }
            set { _active = value; }
        }

        internal MembershipUser CreateMembershipUser(string providerName)
        {
            return new MembershipUser(providerName, this.UserName, this.Key, this.Email, this.ChallengeQuestion,
                    this.Description, this.IsActive, this.IsLocked, this.CreatedOn, this.LastLoggedIn, this.LastActivity,
                    this.LastPasswordChange, this.LastLockedOut);
        }
    }
}
