Ext.namespace('Demo');

/**
 * @class Demo.ProductForm
 * @extends Ext.form.FormPanel
 */
Demo.ProductForm = Ext.extend(Ext.form.FormPanel, {
    // Properties
    title: 'New Product',
    labelWidth: 75,
    border:false,
    bodyStyle: 'padding: 10px;',
    defaults: {
        width: 130
    },
    defaultType: 'textfield',

    // Overriden methods
    initComponent: function() {
        // Stuff in this. are CONFIGS (read-write)
        this.items = [{
            fieldLabel: 'Product SKU',
            name: 'productSku',
            allowBlank:false
        },{
            fieldLabel: 'Price',
            name: 'price'
        }, 
            new Ext.form.DateField({
                fieldLabel: 'Expires',
                name: 'expires'
            })
        ];
        this.buttons = [{
            text: 'Save',
            handler: this.save,
            scope: this
        },{
            text: 'Cancel',
            handler: this.cancel,
            scope: this
        }];
        Demo.ProductForm.superclass.initComponent.call(this);
        // event Names are ALWAYS lowercase
        this.addEvents('save', 'mysecondevent');
        // Stuff in this. are PROPERTIES (read-only)
    },
    
    // Custom Methods
    save: function() {
        var basicForm = this.getForm();
        if (basicForm.isValid()) {
            // this makes our signature
            // - FormPanel
            // - Values
            // eg. myProductForm.on('save', function(form, vals) {
            //    
            //})
            this.fireEvent('save', this, basicForm.getValues());
        }        
    },
    
    cancel: function() {
        
    }
});


/**
 * @class Demo.CategoryTree
 * @extends Ext.tree.TreePanel
 */
Demo.CategoryTree = Ext.extend(Ext.tree.TreePanel, {
    // Properties
    title: 'Categories',

    // Overriden methods
    initComponent: function() {
        this.root = new Ext.tree.TreeNode({
            text: 'Categories',
            expanded: true
        })
        Demo.CategoryTree.superclass.initComponent.call(this);        
        this.root.appendChild(new Ext.tree.TreeNode({text: 'Widgets'}));
        this.root.appendChild(new Ext.tree.TreeNode({text: 'Utilities'}));
        this.root.appendChild(new Ext.tree.TreeNode({text: 'Toys'}));			        
    }
    
    // Custom Methods
});
Ext.reg('categorytree', Demo.CategoryTree);


/**
 * @class Demo.ProductGrid
 * @extends Ext.grid.GridPanel
 */
Demo.ProductGrid = Ext.extend(Ext.grid.GridPanel, {
    // Properties
    title: 'Product Inventory',
    viewConfig: {
        forceFit: true
    },

    // Overriden methods
    initComponent: function() {
        this.columns = [{
                dataIndex: 'productSku',
                header: 'Product SKU'
        },{
                dataIndex: 'price',
                header: 'Price',
                renderer: Ext.util.Format.usMoney
        },{
                dataIndex: 'expires',
                header: 'Expires',
                renderer: Ext.util.Format.dateRenderer('m/d/Y')
        }];
        Demo.ProductGrid.superclass.initComponent.call(this);
    }
    // Custom Methods
});

/**
 * @class Demo.ProductStore
 * @extends Ext.data.SimpleStore
 */
Demo.ProductStore = Ext.extend(Ext.data.SimpleStore, {
    constructor: function(config) {
        config = config || {};
        config.fields = [
						'productSku',
						'price', 
						{name: 'expires', type: 'date', dateFormat: 'm-d-Y'}
					]; // fields go here...
        config.data = [
						['A0015', 101.25, '09-01-2007'],
						['C0084', 29.01, '09-02-2007'],
						['H0016', 83.81, '09-05-2007']
					];  // data goes here...
        Demo.ProductStore.superclass.constructor.call(this, config);
    }
});


/**
 * @class Demo.App
 * @extends Ext.Panel
 */
Demo.App = Ext.extend(Ext.Panel, {
    // Properties
    layout: 'border',
    
    // Overriden methods
    initComponent: function() {
        this.grid = new Demo.ProductGrid({
            itemId: 'grid',
            store: new Demo.ProductStore()
        });
        var form = new Demo.ProductForm({itemId: 'form'});
        this.tabs = new Ext.TabPanel({
            border:false,
            resizeTabs:true,
            enableTabScroll:true,
            activeItem:0,
            defaults:{
                autoScroll:true
            },
            items:[this.grid,form]
        });	        
        // three items here...
        this.items =[{
            xtype: 'categorytree',
            region: 'west',
            width: 200,
            split: true,
            title: 'Categories',
            collapsible: true
        },{
                region: 'center',
                title: 'Content',
                layout: 'fit',
                items: [this.tabs]
        },{
                region: 'south',
                title: 'South',
                height: 150,
                collapsible: true,
                collapsed: true
        }];
        Demo.App.superclass.initComponent.call(this);
        form.on('save', this.onFormSave, this);
    },
    
    // Custom Methods
    onFormSave: function(form, vals) {
        var ds = this.grid.getStore();
        var Product = ds.recordType;
        var r = new Product(vals);
        ds.add(r);
        this.tabs.setActiveTab('grid');
    }
});
Ext.reg('app', Demo.App);


